/* -*-C-*-
*******************************************************************************
*
* File:         EnhanceMail.m
* RCS:          /usr/local/sources/CVS/EnhanceMail/EnhanceMail.m,v 1.5 1997/05/01 00:12:38 tom Exp
* Description:  
* Author:       Carl Edman
* Created:      Fri Oct 13 11:48:05 1995
* Modified:     Sun Apr 13 22:30:48 1997 Tom Hageman <tom@basil.icce.rug.nl>
* Language:     C
* Package:      N/A
* Status:       Experimental (Do Not Distribute)
*
* (C) Copyright 1995, but otherwise this file is perfect freeware.
*
*******************************************************************************
*/

#import "EnhanceMail.h"
#import "Preferences.h"
#import "TransferPanel.h"
#import <ctype.h>
#import <sys/wait.h>

NXBundle *EnhanceBundle=0;
char *EnhanceVersion="X";
char *regerrval=0;

static const char *CVSName="EMrelease2_0b6";
static BOOL windowsOnUnhide=NO;

BOOL EnhanceControlP(void)
{
   NXEvent *ev=[NXApp currentEvent];
   return ev && (ev->flags & NX_CONTROLMASK);
}

BOOL EnhanceShiftP(void)
{
   NXEvent *ev=[NXApp currentEvent];
   return ev && (ev->flags & NX_SHIFTMASK);
}

BOOL EnhanceAlternateP(void)
{
   NXEvent *ev=[NXApp currentEvent];
   return ev && (ev->flags & NX_ALTERNATEMASK);
}

BOOL EnhancePaths(const char *paths[])
{
   int i;
   char path[MAXPATHLEN+1];
   
   if (EnhanceBundle==0)
      EnhanceBundle=[NXBundle bundleForClass:[EnhanceMail class]];
   
   for(i=0;paths[i];i++)
   {
      if ((paths[i][0]=='~') && (paths[i][1]=='/'))
      {
         sprintf(path,"%s%s",NXHomeDirectory(),paths[i]+1);
         paths[i]=NXUniqueString(path);
      }
      else if ((paths[i][0]=='#') && (paths[i][1]=='/'))
      {
         sprintf(path,"%s%s",[EnhanceBundle directory],paths[i]+1);
         paths[i]=NXUniqueString(path);
      }
   }

   return YES;
}

// Try to determine whether to use KANJI quoting at runtime.
static BOOL use_kanji()
{
   const char *const *languages=[NXApp systemLanguages];

   if (languages && languages[0])
   {
      if (strcasecmp(languages[0],"Japanese")==0) return YES;
   }
   return NO;
}

BOOL EnhanceUseKanji(void)
{
   // Cache result of use_kanji().
   static int _use_kanji = -1;

   if (_use_kanji < 0) _use_kanji = use_kanji();
   return (BOOL)_use_kanji;
}

void regerror(char *s)
{
   [NXApp logError:"regexp(3): %s",s];
}

@implementation EnhanceMail

+ finishLoading:(struct mach_header *)header
{
   const char *beg=CVSName,*end=CVSName+strlen(CVSName);
   
   [self poseAs:[self superclass]];
   [NXApp changeClassTo:[self class]];

   while((beg<=end) && !isdigit(*beg)) beg++;
   while((beg<=end) && !isdigit(*end)) end--;
   
   if (beg<end)
   {
      char *c;

      end++;
      EnhanceVersion=strncpy(malloc(end-beg+1),beg,end-beg);
      EnhanceVersion[end-beg]='\0';
      for(c=EnhanceVersion;*c;c++) if (*c=='_') *c='.';
   }
   
   return self;
}

- (void)_maybeShowPanels
{
   Window *origMainWindow = [NXApp mainWindow];

   if (EnhanceLaunchMailboxes) [(MailDriver *)NXApp mailboxes:self];
   if (EnhanceLaunchAddresses) [(MailDriver *)NXApp address:self];
   [origMainWindow makeKeyAndOrderFront:self];
}

- initialCheck:sender
{
   if ([self isHidden])
   {
      windowsOnUnhide=YES;
   }
   else
   {
      [self _maybeShowPanels];
   }
   return [super initialCheck:sender];
}

- appDidUnhide:sender
{
   if (windowsOnUnhide)
   {
      [self _maybeShowPanels];
      windowsOnUnhide=NO;
   }
   return [super appDidUnhide:sender];
}

- (BOOL)updateCell:cell
{
   return [super updateCell:cell];
}

- checkNewMail:(const char *)fp16 beep:(BOOL)fp20
{
   id ret = [super checkNewMail:fp16 beep:fp20];
   [[TransferPanel new] updateBrowser];
   return ret;
}

@end // EnhanceMail


@implementation Menu(EnhanceMail)

- findCellWithTitle:(const char *)title
{
   id cell=nil;
   int x, y, rows, cols;
   const char *c,*end;

   if (title==0) return nil;
   for(end=title;(*end) && (*end!='/');end++);
   
   [matrix getNumRows:&rows numCols:&cols];
   for(y=0;(y<rows) && (cell==nil);y++) for(x=0;(x<cols) && (cell==nil);x++)
   {
      if ((cell=[matrix cellAt:y:x])==nil) continue;
      
      if (((c=[cell title])==0)||(strncmp(title,c,end-title)!=0)||(c[end-title]!='\0'))
         cell=nil;
   }

   if (cell==nil)
      return nil;
   else if ((*end!='\0') && [cell hasSubmenu])
      return [[cell target] findCellWithTitle:end+1];
   else if (*end=='\0')
      return cell;
   else
      return nil;
}

- addSlashItem:(const char *)title action:(SEL)sel keyEquivalent:(unsigned short)code
{
   id cell=nil;
   int x, y, rows, cols;
   const char *c,*end;

   if (title==0) return nil;
   for(end=title;(*end) && (*end!='/');end++);

   [matrix getNumRows:&rows numCols:&cols];
   for(y=0;(y<rows) && (cell==nil);y++) for(x=0;(x<cols) && (cell==nil);x++)
   {
      if ((cell=[matrix cellAt:y:x])==nil) continue;
      
      if (((c=[cell title])==0)||(strncmp(title,c,end-title)!=0)||(c[end-title]!='\0'))
         cell=nil;
   }

   if ((cell==nil) && (*end=='\0'))
   {
      cell=[self addItem:title action:sel keyEquivalent:code];
      [self display];
      return cell;
   }
   else if (cell==nil)
   {
      char *d=alloca(end-title+1);
      strncpy(d,title,end-title);
      d[end-title]='\0';
      cell=[self addItem:d action:0 keyEquivalent:0];
      [self setSubmenu:[[Menu alloc] initTitle:d] forItem:cell];
      [self display];
      return [[cell target] addSlashItem:end+1 action:sel keyEquivalent:code];
   }
   else if ((*end!='\0') && [cell hasSubmenu])
      return [[cell target] addSlashItem:end+1 action:sel keyEquivalent:code];
   else if (*end=='\0')
      return cell;
   else
      return nil;
}

@end // Menu(EnhanceMail)


@implementation Object(EnhanceMail)

- changeClassTo:class
{
   /* Don't try this at home, kids */
   isa=class;
   return self;
}

@end // Object(EnhanceMail)


@implementation ButtonCell(EnhanceMail)

- (int)state
{
   return [self intValue];
}

- setState:(int)anInt
{
   return [self setIntValue:anInt];
}

@end // ButtonCell(EnhanceMail)
