/* -*-C-*-
*******************************************************************************
*
* File:         MailReader.m
* RCS:          /usr/local/sources/CVS/EnhanceMail/MailReader.m,v 1.6 1997/05/08 01:41:41 tom Exp
* Description:  
* Author:       Carl Edman
* Created:      Fri Oct 13 11:48:05 1995
* Modified:     Sat Apr 13 19:09:44 1996 (Carl Edman) cedman@capitalist.princeton.edu
* Language:     C
* Package:      N/A
* Status:       Experimental (Do Not Distribute)
*
* (C) Copyright 1995, but otherwise this file is perfect freeware.
*
*******************************************************************************
*/

#import "EnhanceMail.h"
#import "MailReader.h"
#import "MailBox.h"
#import "XFace.h"
#import "XImageURL.h"
#import "Preferences.h"
#import "TransferPanel.h"

#define MENU_MESSAGE_FLAG NXLocalizedStringFromTableInBundle("Localizable", EnhanceBundle, "Message/Flag", NULL, Slashed menu path to flag command)
#define MENU_MESSAGE_STRIP NXLocalizedStringFromTableInBundle("Localizable", EnhanceBundle, "Message/Strip", NULL, Slashed menu path to strip command)
#define MENU_MAILBOX_INDEX NXLocalizedStringFromTableInBundle("Localizable", EnhanceBundle, "Mailbox/Index", NULL, Slashed menu path to create index command)
#define MENU_MAILBOX_UNINDEX NXLocalizedStringFromTableInBundle("Localizable", EnhanceBundle, "Mailbox/Delete index", NULL, Slashed menu path to remove index command)
#define STATUS_INDEXED NXLocalizedStringFromTableInBundle("Localizable", EnhanceBundle, "indexed", NULL, in MailReader status field)

static id index_menucell=nil;
static id indextag_image=nil;

@implementation EnhanceMailReader

+ finishLoading:(struct mach_header *)header
{
   char path[MAXPATHLEN+1];
   
   if (EnhanceBundle==0) EnhanceBundle=[NXBundle bundleForClass:[self class]];
   [self poseAs:[self superclass]];
   [[NXApp mainMenu] addSlashItem:MENU_MESSAGE_FLAG action:@selector(toggleFlagged:) keyEquivalent:0];
#if 0
   [[NXApp mainMenu] addSlashItem:MENU_MESSAGE_STRIP action:@selector(strip:) keyEquivalent:0];
#endif
   index_menucell=[[NXApp mainMenu] addSlashItem:MENU_MAILBOX_INDEX action:@selector(toggleIndexing:) keyEquivalent:0];

   if ([EnhanceBundle getPath:path forResource:"indexTag" ofType:"tiff"])
      indextag_image=[[NXImage alloc] initFromFile:path];
   
   return self;
}

- loadMessage:(int)fp16 newMsg:(BOOL)fp20 withAllHeaders:(BOOL)fp24
{
   const char *header;
   id ret=[super loadMessage:fp16 newMsg:fp20 withAllHeaders:fp24];
   if (EnhanceShowXImageURL
       && (header=[mailMessage headerValueForKey:"X-Image-URL"]))
      [command setXImageURL:header];
   if (EnhanceShowXFace
       && (header=[mailMessage headerValueForKey:"X-Face"]))
      [command setXFace:header];
   //[command perform:@selector(setXImageURL:) with:nil afterDelay:3000 cancelPrevious:YES];
   return ret;
}

- (void)updateTransferPanel:(int)oldMessageCount
{
   //printf("oldMessageCount:%d\n", oldMessageCount);
   //printf("newMessageCount:%d\n", newMessageCount);
   if (newMessageCount != oldMessageCount)
   {
      //printf("updating Transferpanel for %s\n",[mailbox dirname]);
      [[TransferPanel new] updateMessageCount:newMessageCount old:oldMessageCount
       forMailBoxNamed:[mailbox dirname] isNew:NO];
   }
}

#if 0 // Never worked, since super does not change newMessageCount.
- haveNewMail
{
   int oldMessageCount = newMessageCount;
   id ret = [super haveNewMail];
   //printf("*****haveNewMail:%p\n",ret);
   //printf("newMessageCount=%d\n",newMessageCount);
   [[TransferPanel new] updateMessageCount:newMessageCount old:oldMessageCount
    forMailBoxNamed:[mailbox dirname] isNew:YES];
   return ret;
}
#endif

- free
{
   if (EnhanceAutoCompact && mailbox)
   {
      struct u_mailboxstatus stat;
      [mailbox getMailboxStatus:&stat];
      if (stat.delmsgno>0) [self compact:self];
   }
   return [super free];
}

- compact:sender
{
   if (mailbox && [mailbox hasStore])
   {
      id ret;
      [self toggleIndexing:sender];
      ret=[super compact:sender];
      [self toggleIndexing:sender];
      return ret;
   }
   return [super compact:sender];
}

- strip:sender
{
#if 0
   id list=[matrix getSelectedCells:nil];
   id cell;
   int i;

   for(i=0;i<[list count];i++)
   {
      cell=[list objectAt:i];
      if (cell==nil) continue;
      /* XXX Strip every message of all NeXT/MIME/uuencoded attachments XXX */
   }
   
   list=[list free];
#endif
   return self;
}

- (void)setFlagged:(BOOL)flag
{
   id cell=[matrix selectedCell];
   if (cell==nil) return;
   [cell setFlagged:YES];
   if (mailbox==nil) return;
   [mailbox setState:'+' forMsg:[cell msgId] flush:NO];
   [self display];
   return;
}

- toggleIndexing:sender
{
   if (mailbox==nil) return nil;
   if ([mailbox hasStore])
      [mailbox deleteStore];
   else
      [mailbox createStore];

   [self display];
   [self windowDidBecomeMain:self];
   [self updateStatus];
   return self;
}

- windowDidBecomeMain:sender
{
   const char *c;

   if ((mailbox==nil)||(index_menucell==nil)) return self;
   
   c=[mailbox hasStore] ? MENU_MAILBOX_UNINDEX : MENU_MAILBOX_INDEX;
   if (c=rindex(c,'/')) [index_menucell setTitle:(c+1)];
   return self;
}

- (void)updateStatus
{
   const char *val, *indexed;
   char buf[MAXPATHLEN+1];
   float size;
   int oldMessageCount = newMessageCount;
   
   [super updateStatus];
   [self updateTransferPanel:oldMessageCount];

   if ((statusField==nil)
       ||(mailbox==nil)||((size=[mailbox storeSize])<0)) return;

   val=[statusField stringValue];
   indexed = STATUS_INDEXED;

   if (strstr(val,"MB") || strstr(val,"KB"))
   {
      if (size/(1<<10) > 750.0)
	 sprintf(buf,"%s \320 %s (%.1fMB)",val,indexed,size/(1<<20));
      else
	 sprintf(buf,"%s \320 %s (%.0fKB)",val,indexed,size/(1<<10));
   }
   else
      sprintf(buf,"%s \320 %s",val,indexed);
   
   [statusField setStringValue:buf];
}

- drawSelf:(const NXRect *)rects :(int)rectCount
{
   id ret=[super drawSelf:rects:rectCount];
   NXPoint p;
   NXSize s;
   if ((mailbox==nil)||(indextag_image==nil)) return nil;
   
   [indextag_image getSize:&s];
   p.x=0; p.y=s.height;
   if ([mailbox hasStore]) [indextag_image composite:NX_SOVER toPoint:&p];
   return ret;
}

#if 0 // Experimental cruft.
// flagged messages loose their flag when transferred
- showTransferMsg:(int)fp16 to:(const char *)fp20;
{
   //printf("showTransferMsg:%d,%s\n",fp16,fp20);
   return [super showTransferMsg:fp16 to:fp20];
}
- transfer:fp16;
{
   //printf("transfer:%p class=%s\n",fp16,[[fp16 class] name]);
   return [super transfer:fp16];
}
- showTransfer:fp16;
{
   //printf("showTransfer:%p\n",fp16);
   return [super showTransfer:fp16];
}
#endif

@end // EnhanceMailReader


@implementation CommandView(EnhanceMailReaderFace)

- setXFace:(const char *)xface
{
   NXSize size;
   
   if (face) return self;
   size.width=size.height=64;
   face=[[NXImage alloc] initXFace:xface size:&size];
   [self display];
   return self;
}

- setXImageURL:(const char *)ximageurl
{
   if (face) return self;
   face=[[NXImage alloc] initURL:ximageurl];
   [self display];
   return self;
}

@end // CommandView (EnhanceMailReader)
